#include <stdlib.h>
#include <string.h>
#include <qdos.h>
#include "q40snd.h"
#include "version.h"

char _prog_name[] = "play " VERSION;
char *_endmsg = NULL;

static int maxio = MAXIO;
static int stdchn = 0x00010001;
static short verbose = 0;
static int factor = 800;

void showvalue (char *a, int *b, int nl)
{
    char foo[128], *p = foo;
    int ns;

    if (a)
    {
	p = stpcpy (foo, a);
    }
    if (b)
    {
	itoa (*b, p);
	ns = strlen (p);
	p = p + ns;
    }
    if (nl)
    {
	*p++ = '\n';
    }
    ns = p - foo;
    if (ns)
	io_sstrg (stdchn, -1, foo, ns);
}

#ifndef UQLX_EMUSOUND
# define SNDBUSY(_p1) sndbusy()
#else
# define SNDBUSY(__p1) snd_busy(__p1)

static int fd = 0;

int snd_busy (int n)
{
    n -= factor;
    /* Note: Samples in queue is 1/2 no bytes !! */
    return (n > 0) ? n >> 1 : 0;
}

int sndplay (void *buf, int n)
{

    if (fd == 0)
    {
	fd = io_open ("win2_tmp_.snd", 3);
    }
    if (fd > 0)
    {
	int m, k;

	m = MIN (n, maxio);
	k = io_sstrg (fd, -1, buf, (short) m);
	if (k > 0)
	{
	    n -= k;
	}
	else
	    n = 0;
    }
    else
    {
	showvalue ("Fails ", &fd, 1);
    }
    return n;
}

#endif

int makenoise (void *buf, int nsz)
{
    int j, n, res = 0;

    do
    {
	n = sndplay (buf, nsz);
	if (verbose)
	    showvalue (" Returning ", &n, 0);
	if (n < 0)
	{
	    res = n;
	    break;
	}
	else if (n)
	{
	    int nl = nsz - n;

	    j = SNDBUSY (nl);
	    if (verbose)
		showvalue (" Busy ", &j, 1);
	    j /= factor;
	    if (j > 0)
		mt_susjb (-1, j, 0);
	    (char *) buf = (char *) buf + nl;
	    nsz = n;
	}
    }
    while (n);

    return res;
}

/* Simpe strtol for base 10, +ve number, optional leading space */

int myatoi (char **p)
{
    int n = 0;
    while (**p == ' ')
	(*p)++;
    while (**p >= '0' && **p <= '9')
    {
	n = n * 10 + ((**p) - '0');
	(*p)++;
    }
    return n;
}


int fnread (int ch, void *buf, int nsz)
{
    int n, k, l;
    int m;

    l = 0;
    while (nsz > 0)
    {
	m = MIN (nsz, maxio);
	k = io_fstrg (ch, -1, buf, (short) m);

	if (k > 0)
	{
	    nsz -= k;
	    (char *) buf = (char *) buf + k;
	    l += k;
	}
	else
	{
	    nsz = 0;
	    if (k != -10)
		l = k;
	}
    }
    return l;
}

int main (int ac, char **av)
{
    int rn = ERR_BP, ch;
    char *sp;
    short nc;
    char *ep;
    int prio = -1;

    sp = *(av + 1);

    nc = *((short *) sp);
    sp += 2;

    if (nc)
    {
	stdchn = *(long *) sp;
    }

    sp += ((nc << 2));
    if ((nc = *((short *) sp)))
    {
	char args[128];
	struct qdirect d;
	char *buf, *fnam;

	memcpy (args, sp + 2, nc);
	*(args + nc) = 0;
	for (fnam = args; *fnam; fnam++)
	{
	    int f;

	    if (*fnam == ' ')
		continue;
	    else if (*fnam == '-')
	    {
		*fnam++;
		switch (*fnam)
		{
		    case 'v':
			verbose = 1;
			break;
		    case 'm':
			fnam++;
			{
			    int m;
			    m = myatoi (&fnam);
			    if (*fnam == 'k' || *fnam == 'K')
			    {
				fnam++;
				m = m << 10;
			    }

			    if (m > 1024 && m <= MAXIO)
			    {
				maxio = m;
			    }
                            maxio &= ~1;
			}
			break;
		    case 'p':
			fnam++;
			prio = myatoi (&fnam);
			if (prio > 0 && prio < 128)
			    mt_prior (-1, prio);
			break;
		    case 'f':
			fnam++;
			f = myatoi (&fnam);
			if (f > 400 && f <= 10000)
			    factor = f;
			break;
		}
	    }
	    else
		break;
	}

	*(fnam + nc) = 0;

	showvalue (fnam, 0, 0);
	showvalue (", factor=", &factor, 0);
	showvalue (", maxio=", &maxio, 0);
	showvalue (", prio=", &prio, 1);

#if 0				// Sanity tests
	{
	    int bp;
	    bp = sndplay (0, 0);
	    showvalue ("Bad param 0 = ", &bp, 1);
	    bp = sndplay (0, -99);
	    showvalue ("Bad param -ve = ", &bp, 1);
	    return bp;
	}
#endif
	if ((ch = io_open (fnam, 0)) > 0)
	{
	    if ((rn = fs_headr (ch, -1, &d, sizeof (d))) > 0)
	    {
		int maxb;
		maxb = MIN (d.d_length, 200 * 1024);
		maxb &= ~1;
		rn = 0;

		if ((buf = malloc (maxb)) != NULL)
		{
		    int n;
		    while (rn == 0 && (n = fnread (ch, buf, maxb)) > 0)
		    {
			n &= ~1;
			if (verbose)
			{
			    showvalue ("+Read ", &n, 1);
			}
			rn = makenoise (buf, n);
			if (verbose)
			{
			    showvalue ("\n-Done ", &rn, 1);
			}
		    }
		    if (n < 0)
			rn = n;
		}
		else
		    rn = ERR_OM;
	    }
	    io_close (ch);
	}
	else
	    rn = ch;
    }
    return rn;
}
int (*_Cstart) (int, char **) = main;
